from pydantic import BaseModel, SecretStr, BaseSettings


class BaseServiceSettings(BaseSettings):

    class Config:
        env_file = ".env"
        env_nested_delimiter = "__"


class MongoSettings(BaseModel):
    """
    All Mongo settings, including connection settings
    """
    username: str
    password: SecretStr
    host: str
    port: int = 27017
    database: str = "testable"
    params: dict[str, str] = {"ssl": "false"}

    @property
    def uri(self) -> str:
        """Returns the formatted URI from the settings"""
        uri = f"mongodb://{self.username}:{self.password.get_secret_value()}@{self.host}:{self.port}/"
        if self.params:
            uri += "?"

        uri += "&".join(k + "=" + v for k, v in self.params.items())
        return uri



from fastapi import FastAPI
from pymongo import MongoClient
from pymongo.database import Database
from typing import Any, TypeVar
from pydantic import BaseModel

from .settings import BaseServiceSettings, MongoSettings


class WithMongo(BaseModel):
    """
    The attributes that get added on for the Mongo Mixin
    """
    mongo: MongoClient
    database: Database


class TestableApp(FastAPI):
    """
    A starter generic app for all Testable microservices.
    """
    __S = TypeVar("__S", bound=BaseServiceSettings)

    def __init__(self, settings: __S, **extra: Any):
        """
        Initializes a new FastAPI app with some Pydantic settings.

        Parameters
        ----------
        settings : _SettingsType
            Any `BaseServiceSettings` type settings to initialize app with

        extra : Any
            Additional kwargs will be passed to FastAPI
        """
        super().__init__(**extra)
        self.settings = settings

    __T = TypeVar("__T")

    def with_mongo(self: __T, settings: MongoSettings) -> __T | WithMongo:
        """
        Initializes a Mongo connection and binds the client and the Testable database to the app instance.

        Parameters
        ----------
        settings : MongoSettings
            Mongo Settings to use to initialize the connection

        Returns
        -------
        app : __T | WithMongo
            Will return the current app with the Mongo mixin
        """
        setattr(self, "mongo", MongoClient(settings.uri, uuidRepresentation="standard"))
        setattr(self, "database", getattr(self, "mongo")[settings.database])
        app = self
        return app



from setuptools import setup, find_packages

VERSION = "0.2.8"
DESCRIPTION = 'Core Testable Service Modules'
LONG_DESCRIPTION = 'Core service modules that provide access to shared functionality across Testable services.'

# Setting up
setup(
    name="testable_coresvc",
    version=VERSION,
    author="Ryan de Marigny",
    author_email="testableproject@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=["pydantic>=1.10.2", "typing_extensions>=4.4.0", "fastapi>=0.87.0", "pymongo>=4.3.3"],
    keywords=['python', 'first package'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
